@@ -7,73 +7,112 @@
 #
 # References:   1) http://park2.wakwak.com/~tsuruzoh/Computer/Digicams/exif-e.html
 #               2) Christian Koller private communication (tests with the 20D)
+#               3) Rainer Honle private communication (tests with the 5D)
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::CanonCustom;
 
 use strict;
 use vars qw($VERSION);
+use Image::ExifTool::Canon;
 
-$VERSION = '1.06';
+$VERSION = '1.09';
 
 sub ProcessCanonCustom($$$);
 sub WriteCanonCustom($$$);
 sub CheckCanonCustom($$$);
+sub ConvertPFn($);
+sub ConvertPFnInv($);
+
+my %onOff = ( 0 => 'On', 1 => 'Off' );
+my %offOn = ( 0 => 'Off', 1 => 'On' );
+my %disableEnable = ( 0 => 'Disable', 1 => 'Enable' );
+my %enableDisable = ( 0 => 'Enable', 1 => 'Disable' );
+my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
 
 #------------------------------------------------------------------------------
-# Custom functions for the D30/D60
+# Custom functions for the 1D
 # CanonCustom (keys are custom function number)
-%Image::ExifTool::CanonCustom::FunctionsD30 = (
+%Image::ExifTool::CanonCustom::Functions1D = (
     GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
     PROCESS_PROC => \&ProcessCanonCustom,
     WRITE_PROC => \&WriteCanonCustom,
     CHECK_PROC => \&CheckCanonCustom,
     WRITABLE => 'int8u',
-    1 => {
-        Name => 'LongExposureNoiseReduction',
+    0 => {
+        Name => 'FocusingScreen',
         PrintConv => {
-            0 => 'Off',
-            1 => 'On',
+            0 => 'Ec-N, R',
+            1 => 'Ec-A,B,C,CII,CIII,D,H,I,L',
         },
     },
+    1 => {
+        Name => 'FinderDisplayDuringExposure',
+        PrintConv => \%offOn,
+    },
     2 => {
-        Name => 'Shutter-AELock',
+        Name => 'ShutterReleaseNoCFCard',
+        Description => 'Shutter Release W/O CF Card',
         PrintConv => {
-            0 => 'AF/AE lock',
-            1 => 'AE lock/AF',
-            2 => 'AF/AF lock',
-            3 => 'AE+release/AE+AF',
+            0 => 'Yes',
+            1 => 'No',
         },
     },
     3 => {
-        Name => 'MirrorLockup',
+        Name => 'ISOSpeedExpansion',
+        Description => 'ISO Speed Expansion',
         PrintConv => {
-            0 => 'Disable',
-            1 => 'Enable',
+            0 => 'No',
+            1 => 'Yes',
         },
     },
     4 => {
-        Name => 'ExposureLevelIncrements',
+        Name => 'ShutterAELButton',
+        Description => 'Shutter Button/AEL Button',
         PrintConv => {
-            0 => '1/2 Stop',
-            1 => '1/3 Stop',
+            0 => 'AF/AE lock stop',
+            1 => 'AE lock/AF',
+            2 => 'AF/AF lock, No AE lock',
+            3 => 'AE/AF, No AE lock',
         },
     },
     5 => {
-        Name => 'AFAssist',
+        Name => 'ManualTv',
+        Description => 'Manual Tv/Av For M',
         PrintConv => {
-            0 => 'Auto',
-            1 => 'Off',
+            0 => 'Tv=Main/Av=Control',
+            1 => 'Tv=Control/Av=Main',
+            2 => 'Tv=Main/Av=Main w/o lens',
+            3 => 'Tv=Control/Av=Main w/o lens',
         },
     },
     6 => {
-        Name => 'FlashSyncSpeedAv',
+        Name => 'ExposureLevelIncrements',
         PrintConv => {
-            0 => 'Auto',
-            1 => '1/200 Fixed',
+            0 => '1/3-stop set, 1/3-stop comp.',
+            1 => '1-stop set, 1/3-stop comp.',
+            2 => '1/2-stop set, 1/2-stop comp.',
         },
     },
     7 => {
+        Name => 'USMLensElectronicMF',
+        PrintConv => {
+            0 => 'Turns on after one-shot AF',
+            1 => 'Turns off after one-shot AF',
+            2 => 'Always turned off',
+        },
+    },
+    8 => {
+        Name => 'LCDPanels',
+        Description => 'Top/Back LCD Panels',
+        PrintConv => {
+            0 => 'Remain. shots/File no.',
+            1 => 'ISO/Remain. shots',
+            2 => 'ISO/File no.',
+            3 => 'Shots in folder/Remain. shots',
+        },
+    },
+    9 => {
         Name => 'AEBSequence',
         PrintConv => {
             0 => '0,-,+/Enabled',
@@ -82,70 +121,246 @@ sub CheckCanonCustom($$$);
             3 => '-,0,+/Disabled',
         },
     },
-    8 => {
+    10 => {
+        Name => 'AFPointIllumination',
+        PrintConv => {
+            0 => 'On',
+            1 => 'Off',
+            2 => 'On without dimming',
+            3 => 'Brighter',
+        },
+    },
+    11 => {
+        Name => 'AFPointSelection',
+        PrintConv => {
+            0 => 'H=AF+Main/V=AF+Command',
+            1 => 'H=Comp+Main/V=Comp+Command',
+            2 => 'H=Command only/V=Assist+Main',
+            3 => 'H=FEL+Main/V=FEL+Command',
+        },
+    },
+    12 => {
+        Name => 'MirrorLockup',
+        PrintConv => \%disableEnable,
+    },
+    13 => {
+        Name => 'AFPointSpotMetering',
+        Description => 'No. AF Points/Spot Metering',
+        PrintConv => {
+            0 => '45/Center AF point',
+            1 => '11/Active AF point',
+            2 => '11/Center AF point',
+            3 => '9/Active AF point',
+        },
+    },
+    14 => {
+        Name => 'FillFlashAutoReduction',
+        PrintConv => \%enableDisable,
+    },
+    15 => {
         Name => 'ShutterCurtainSync',
         PrintConv => {
             0 => '1st-curtain sync',
             1 => '2nd-curtain sync',
         },
     },
-    9 => {
+    16 => {
+        Name => 'SafetyShiftInAvOrTv',
+        PrintConv => \%disableEnable,
+    },
+    17 => {
+        Name => 'AFPointActivationArea',
+        PrintConv => {
+            0 => 'Single AF point',
+            1 => 'Expanded (TTL. of 7 AF points)',
+            2 => 'Automatic expanded (max. 13)',
+        },
+    },
+    18 => {
+        Name => 'SwitchToRegisteredAFPoint',
+        PrintConv => {
+            0 => 'Assist + AF',
+            1 => 'Assist',
+            2 => 'Only while pressing assist',
+        },
+    },
+    19 => {
         Name => 'LensAFStopButton',
         PrintConv => {
-            0 => 'AF Stop',
-            1 => 'Operate AF',
-            2 => 'Lock AE and start timer',
+            0 => 'AF stop',
+            1 => 'AF start',
+            2 => 'AE lock while metering',
+            3 => 'AF point: M -> Auto / Auto -> Ctr.',
+            4 => 'AF mode: ONE SHOT <-> AI SERVO',
+            5 => 'IS start',
         },
     },
-    10 => {
-        Name => 'FillFlashAutoReduction',
+    20 => {
+        Name => 'AIServoTrackingSensitivity',
         PrintConv => {
-            0 => 'Enable',
-            1 => 'Disable',
-        },
-    },
-    11 => {
-        Name => 'MenuButtonReturn',
-        PrintConv => {
-            0 => 'Top',
-            1 => 'Previous (volatile)',
-            2 => 'Previous',
-        },
-    },
-    12 => {
-        Name => 'SetButtonFunction',
-        PrintConv => {
-            0 => 'Not assigned',
-            1 => 'Change quality',
-            2 => 'Change ISO speed',
-            3 => 'Select parameters',
-        },
-    },
-    13 => {
-        Name => 'SensorCleaning',
-        PrintConv => {
-            0 => 'Disable',
-            1 => 'Enable',
-        },
-    },
-    14 => {
-        Name => 'SuperimposedDisplay',
-        PrintConv => {
-            0 => 'On',
-            1 => 'Off',
-        },
-    },
-    15 => {
-        Name => 'ShutterReleaseNoCFCard',
-        Description => 'Shutter Release W/O CF Card',
-        PrintConv => {
-            0 => 'Yes',
-            1 => 'No',
+            0 => 'Standard',
+            1 => 'Slow',
+            2 => 'Moderately slow',
+            3 => 'Moderately fast',
+            4 => 'Fast',
         },
     },
 );
 
-# custom functions for 10D
+# Custom functions for the 5D (ref 3)
+%Image::ExifTool::CanonCustom::Functions5D = (
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+    PROCESS_PROC => \&ProcessCanonCustom,
+    WRITE_PROC => \&WriteCanonCustom,
+    CHECK_PROC => \&CheckCanonCustom,
+    WRITABLE => 'int8u',
+    0 => {
+        Name => 'FocusingScreen',
+        PrintConv => {
+            0 => 'Ee-A',
+            1 => 'Ee-D',
+            2 => 'Ee-S',
+        },
+    },
+    1 => {
+        Name => 'SetFunctionWhenShooting',
+        PrintConv => {
+            0 => 'Default (no function)',
+            1 => 'Change quality',
+            2 => 'Change Parameters',
+            3 => 'Menu display',
+            4 => 'Image replay',
+        },
+    },
+    2 => {
+        Name => 'LongExposureNoiseReduction',
+        PrintConv => \%offOn,
+    },
+    3 => {
+        Name => 'FlashSyncSpeedAv',
+        PrintConv => {
+            0 => 'Auto',
+            1 => '1/200 Fixed',
+        },
+    },
+    4 => {
+        Name => 'Shutter-AELock',
+        PrintConv => {
+            0 => 'AF/AE lock',
+            1 => 'AE lock/AF',
+            2 => 'AF/AF lock, No AE lock',
+            3 => 'AE/AF, No AE lock',
+        },
+    },
+    5 => {
+        Name => 'AFAssistBeam',
+        PrintConv => {
+            0 => 'Emits',
+            1 => 'Does not emit',
+        },
+    },
+    6 => {
+        Name => 'ExposureLevelIncrements',
+        PrintConv => {
+            0 => '1/3 Stop',
+            1 => '1/2 Stop',
+        },
+    },
+    7 => {
+        Name => 'FlashFiring',
+        PrintConv => {
+            0 => 'Fires',
+            1 => 'Does not fire',
+        },
+    },
+    8 => {
+        Name => 'ISOExpansion',
+        PrintConv => \%offOn,
+    },
+    9 => {
+        Name => 'AEBSequence',
+        PrintConv => {
+            0 => '0,-,+/Enabled',
+            1 => '0,-,+/Disabled',
+            2 => '-,0,+/Enabled',
+            3 => '-,0,+/Disabled',
+        },
+    },
+    10 => {
+        Name => 'SuperimposedDisplay',
+        PrintConv => \%onOff,
+    },
+    11 => {
+        Name => 'MenuButtonDisplayPosition',
+        PrintConv => {
+            0 => 'Previous (top if power off)',
+            1 => 'Previous',
+            2 => 'Top',
+        },
+    },
+    12 => {
+        Name => 'MirrorLockup',
+        PrintConv => \%disableEnable,
+    },
+    13 => {
+        Name => 'AFPointSelectionMethod',
+        PrintConv => {
+            0 => 'Normal',
+            1 => 'Multi-controller direct',
+            2 => 'Quick Control Dial direct',
+        },
+    },
+    14 => {
+        Name => 'ETTLII',
+        Description => 'E-TTL II',
+        PrintConv => {
+            0 => 'Evaluative',
+            1 => 'Average',
+        },
+    },
+    15 => {
+        Name => 'ShutterCurtainSync',
+        PrintConv => {
+            0 => '1st-curtain sync',
+            1 => '2nd-curtain sync',
+        },
+    },
+    16 => {
+        Name => 'SafetyShiftInAvOrTv',
+        PrintConv => \%disableEnable,
+    },
+    17 => {
+        Name => 'AFPointActivationArea',
+        PrintConv => {
+            0 => 'Standard',
+            1 => 'Expanded',
+        },
+    },
+    18 => {
+        Name => 'LCDDisplayReturnToShoot',
+        PrintConv => {
+            0 => 'With Shutter Button only',
+            1 => 'Also with * etc.',
+        },
+    },
+    19 => {
+        Name => 'LensAFStopButton',
+        PrintConv => {
+            0 => 'AF stop',
+            1 => 'AF start',
+            2 => 'AE lock while metering',
+            3 => 'AF point: M -> Auto / Auto -> Ctr.',
+            4 => 'ONE SHOT <-> AI SERVO',
+            5 => 'IS start',
+        },
+    },
+    20 => {
+        Name => 'AddOriginalDecisionData',
+        PrintConv => \%offOn,
+    },
+);
+
+# Custom functions for 10D
 %Image::ExifTool::CanonCustom::Functions10D = (
     GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
     PROCESS_PROC => \&ProcessCanonCustom,
@@ -174,7 +389,7 @@ sub CheckCanonCustom($$$);
         Name => 'FlashSyncSpeedAv',
         PrintConv => {
             0 => 'Auto',
-            1 => '1/200 (Fixed)',
+            1 => '1/200 Fixed',
         },
     },
     4 => {
@@ -182,8 +397,8 @@ sub CheckCanonCustom($$$);
         PrintConv => {
             0 => 'AF/AE lock',
             1 => 'AE lock/AF',
-            2 => 'AF/AF lock, No AE Lock',
-            3 => 'AE/AF, No AE Lock',
+            2 => 'AF/AF lock, No AE lock',
+            3 => 'AE/AF, No AE lock',
         },
     },
     5 => {
@@ -191,9 +406,9 @@ sub CheckCanonCustom($$$);
         Description => 'AF Assist/Flash Firing',
         PrintConv => {
             0 => 'Emits/Fires',
-            1 => 'Does Not Emit/Fires',
-            2 => 'Only Ext. Flash Emits/Fires',
-            3 => 'Emits/Does Not Fire',
+            1 => 'Does not emit/Fires',
+            2 => 'Only ext. flash emits/Fires',
+            3 => 'Emits/Does not fire',
         },
     },
     6 => {
@@ -238,26 +453,19 @@ sub CheckCanonCustom($$$);
     },
     10 => {
         Name => 'SuperimposedDisplay',
-        PrintConv => {
-            0 => 'On',
-            1 => 'Off',
-        },
+        PrintConv => \%onOff,
     },
     11 => {
         Name => 'MenuButtonDisplayPosition',
         PrintConv => {
-            0 => 'Previous (Volatile)',
+            0 => 'Previous (top if power off)',
             1 => 'Previous',
             2 => 'Top',
         },
     },
-
     12 => {
         Name => 'MirrorLockup',
-        PrintConv => {
-            0 => 'Disable',
-            1 => 'Enable',
-        },
+        PrintConv => \%disableEnable,
     },
     13 => {
         Name => 'AssistButtonFunction',
@@ -271,10 +479,7 @@ sub CheckCanonCustom($$$);
     },
     14 => {
         Name => 'FillFlashAutoReduction',
-        PrintConv => {
-            0 => 'Enable',
-            1 => 'Disable',
-        },
+        PrintConv => \%enableDisable,
     },
     15 => {
         Name => 'ShutterCurtainSync',
@@ -285,10 +490,7 @@ sub CheckCanonCustom($$$);
     },
     16 => {
         Name => 'SafetyShiftInAvOrTv',
-        PrintConv => {
-            0 => 'Disable',
-            1 => 'Enable',
-        },
+        PrintConv => \%disableEnable,
     },
     17 => {
         Name => 'LensAFStopButton',
@@ -300,9 +502,7 @@ sub CheckCanonCustom($$$);
     },
 );
 
-
 # Custom functions for the 20D (ref 2)
-# CanonCustom (keys are custom function number)
 %Image::ExifTool::CanonCustom::Functions20D = (
     GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
     PROCESS_PROC => \&ProcessCanonCustom,
@@ -321,10 +521,7 @@ sub CheckCanonCustom($$$);
     },
     1 => {
         Name => 'LongExposureNoiseReduction',
-        PrintConv => {
-            0 => 'Off',
-            1 => 'On',
-        },
+        PrintConv => \%offOn,
     },
     2 => {
         Name => 'FlashSyncSpeedAv',
@@ -338,14 +535,14 @@ sub CheckCanonCustom($$$);
         PrintConv => {
             0 => 'AF/AE lock',
             1 => 'AE lock/AF',
-            2 => 'AF/AF lock no AE lock',
-            3 => 'AE/AF,no AE lock',
+            2 => 'AF/AF lock, No AE lock',
+            3 => 'AE/AF, No AE lock',
         },
     },
     4 => {
         Name => 'AFAssistBeam',
         PrintConv => {
-            0 => 'Emitts',
+            0 => 'Emits',
             1 => 'Does not emit',
             2 => 'Only ext. flash emits',
         },
@@ -366,10 +563,7 @@ sub CheckCanonCustom($$$);
     },
     7 => {
         Name => 'ISOExpansion',
-        PrintConv => {
-            0 => 'Off',
-            1 => 'On',
-        },
+        PrintConv => \%offOn,
     },
     8 => {
         Name => 'AEBSequence',
@@ -382,10 +576,7 @@ sub CheckCanonCustom($$$);
     },
     9 => {
         Name => 'SuperimposedDisplay',
-        PrintConv => {
-            0 => 'On',
-            1 => 'Off',
-        },
+        PrintConv => \%onOff,
     },
     10 => {
         Name => 'MenuButtonDisplayPosition',
@@ -397,10 +588,7 @@ sub CheckCanonCustom($$$);
     },
     11 => {
         Name => 'MirrorLockup',
-        PrintConv => {
-            0 => 'Disable',
-            1 => 'Enable',
-        },
+        PrintConv => \%disableEnable,
     },
     12 => {
         Name => 'AFPointSelectionMethod',
@@ -412,6 +600,7 @@ sub CheckCanonCustom($$$);
     },
     13 => {
         Name => 'ETTLII',
+        Description => 'E-TTL II',
         PrintConv => {
             0 => 'Evaluative',
             1 => 'Average',
@@ -425,116 +614,148 @@ sub CheckCanonCustom($$$);
         },
     },
     15 => {
-        Name => 'SafetyShiftInAVorTV',
-        PrintConv => {
-            0 => 'Disable',
-            1 => 'Enable',
-        },
+        Name => 'SafetyShiftInAvOrTv',
+        PrintConv => \%disableEnable,
     },
     16 => {
         Name => 'LensAFStopButton',
         PrintConv => {
             0 => 'AF stop',
             1 => 'AF start',
-            2 => 'AF lock while metering',
-            3 => 'AF point: M -> Auto / Auto -> ctr.',
+            2 => 'AE lock while metering',
+            3 => 'AF point: M -> Auto / Auto -> Ctr.',
             4 => 'ONE SHOT <-> AI SERVO',
             5 => 'IS start',
         },
     },
     17 => {
         Name => 'AddOriginalDecisionData',
-        PrintConv => {
-            0 => 'Off',
-            1 => 'On',
-        },
+        PrintConv => \%offOn,
     },
 );
 
-# custom functions for the 1D
-%Image::ExifTool::CanonCustom::Functions1D = (
+# Custom functions for the 350D (PH - unconfirmed)
+%Image::ExifTool::CanonCustom::Functions350D = (
     GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
     PROCESS_PROC => \&ProcessCanonCustom,
     WRITE_PROC => \&WriteCanonCustom,
     CHECK_PROC => \&CheckCanonCustom,
     WRITABLE => 'int8u',
     0 => {
-        Name => 'FocusingScreen',
+        Name => 'SetButtonCrossKeysFunc',
         PrintConv => {
-            0 => 'Ec-N, R',
-            1 => 'Ec-A,B,C,CII,CIII,D,H,I,L',
+            0 => 'Normal',
+            1 => 'Set: Quality',
+            2 => 'Set: Parameter',
+            3 => 'Set: Playback',
+            4 => 'Cross keys: AF frame selec.',
         },
     },
     1 => {
-        Name => 'FinderDisplayDuringExposure',
-        PrintConv => {
-            0 => 'Off',
-            1 => 'On',
-        },
+        Name => 'LongExposureNoiseReduction',
+        PrintConv => \%offOn,
     },
     2 => {
-        Name => 'ShutterReleaseNoCFCard',
-        Description => 'Shutter Release W/O CF Card',
+        Name => 'FlashSyncSpeedAv',
         PrintConv => {
-            0 => 'Yes',
-            1 => 'No',
+            0 => 'Auto',
+            1 => '1/200 Fixed',
         },
     },
     3 => {
-        Name => 'ISOSpeedExpansion',
-        Description => 'ISO Speed Expansion',
+        Name => 'Shutter-AELock',
         PrintConv => {
-            0 => 'No',
-            1 => 'Yes',
+            0 => 'AF/AE lock',
+            1 => 'AE lock/AF',
+            2 => 'AF/AF lock, No AE lock',
+            3 => 'AE/AF, No AE lock',
         },
     },
     4 => {
-        Name => 'ShutterAELButton',
-        Description => 'Shutter Button/AEL Button',
+        Name => 'AFAssistBeam',
         PrintConv => {
-            0 => 'AF/AE Lock Stop',
-            1 => 'AE Lock/AF',
-            2 => 'AF/AF Lock, No AE Lock',
-            3 => 'AE/AF, No AE Lock',
+            0 => 'Emits',
+            1 => 'Does not emit',
+            2 => 'Only emits ext. flash',
         },
     },
     5 => {
-        Name => 'ManualTv',
-        Description => 'Manual Tv/Av For M',
+        Name => 'ExposureLevelIncrements',
         PrintConv => {
-            0 => 'Tv=Main/Av=Control',
-            1 => 'Tv=Control/Av=Main',
-            2 => 'Tv=Main/Av=Main w/o Lens',
-            3 => 'Tv=Control/Av=Main w/o Lens',
+            0 => '1/3 Stop',
+            1 => '1/2 Stop',
         },
     },
     6 => {
-        Name => 'ExposureLevelIncrements',
-        PrintConv => {
-            0 => '1/3-Stop Set, 1/3-Stop Comp',
-            1 => '1-Stop Set, 1/3-Stop Comp',
-            2 => '1/2-Stop Set, 1/2-Stop Comp',
-        },
+        Name => 'MirrorLockup',
+        PrintConv => \%disableEnable,
     },
     7 => {
-        Name => 'USMLensElectronicMF',
+        Name => 'ETTLII',
+        Description => 'E-TTL II',
         PrintConv => {
-            0 => 'Turns On After One-Shot AF',
-            1 => 'Turns Off After One-Shot AF',
-            2 => 'Always Turned Off',
+            0 => 'Evaluative',
+            1 => 'Average',
         },
     },
     8 => {
-        Name => 'LCDPanels',
-        Description => 'Top/Back LCD Panels',
+        Name => 'ShutterCurtainSync',
         PrintConv => {
-            0 => 'Remain. Shots/File No.',
-            1 => 'ISO/Remain. Shots',
-            2 => 'ISO/File No.',
-            3 => 'Shots In Folder/Remain. Shots',
+            0 => '1st-curtain sync',
+            1 => '2nd-curtain sync',
         },
     },
-    9 => {
+);
+
+# Custom functions for the D30/D60
+%Image::ExifTool::CanonCustom::FunctionsD30 = (
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+    PROCESS_PROC => \&ProcessCanonCustom,
+    WRITE_PROC => \&WriteCanonCustom,
+    CHECK_PROC => \&CheckCanonCustom,
+    NOTES => 'Custom functions for the EOS-D30 and D60.',
+    WRITABLE => 'int8u',
+    1 => {
+        Name => 'LongExposureNoiseReduction',
+        PrintConv => \%offOn,
+    },
+    2 => {
+        Name => 'Shutter-AELock',
+        PrintConv => {
+            0 => 'AF/AE lock',
+            1 => 'AE lock/AF',
+            2 => 'AF/AF lock',
+            3 => 'AE+release/AE+AF',
+        },
+    },
+    3 => {
+        Name => 'MirrorLockup',
+        PrintConv => \%disableEnable,
+    },
+    4 => {
+        Name => 'ExposureLevelIncrements',
+        PrintConv => {
+            0 => '1/2 Stop',
+            1 => '1/3 Stop',
+        },
+    },
+    5 => {
+        Name => 'AFAssist',
+        PrintConv => {
+            0 => 'Emits/Fires',
+            1 => 'Does not emit/Fires',
+            2 => 'Only ext. flash emits/Fires',
+            3 => 'Emits/Does not fire',
+        },
+    },
+    6 => {
+        Name => 'FlashSyncSpeedAv',
+        PrintConv => {
+            0 => 'Auto',
+            1 => '1/200 Fixed',
+        },
+    },
+    7 => {
         Name => 'AEBSequence',
         PrintConv => {
             0 => '0,-,+/Enabled',
@@ -543,105 +764,193 @@ sub CheckCanonCustom($$$);
             3 => '-,0,+/Disabled',
         },
     },
-    10 => {
-        Name => 'AFPointIllumination',
-        PrintConv => {
-            0 => 'On',
-            1 => 'Off',
-            2 => 'On Without Dimming',
-            3 => 'Brighter',
-        },
-    },
-    11 => {
-        Name => 'AFPointSelection',
-        PrintConv => {
-            0 => 'H=AF+Main/V=AF+Command',
-            1 => 'H=Comp+Main/V=Comp+Command',
-            2 => 'H=Command Only/V=Assist+Main',
-            3 => 'H=FEL+Main/V=FEL+Command',
-        },
-    },
-    12 => {
-        Name => 'MirrorLockup',
-        PrintConv => {
-            0 => 'Disable',
-            1 => 'Enable',
-        },
-    },
-    13 => {
-        Name => 'AFPointSpotMetering',
-        Description => 'No. AF Points/Spot Metering',
-        PrintConv => {
-            0 => '45/Center AF Point',
-            1 => '11/Active AF Point',
-            2 => '11/Center AF Point',
-            3 => '9/Active AF Point',
-        },
-    },
-    14 => {
-        Name => 'FillFlashAutoReduction',
-        PrintConv => {
-            0 => 'Enable',
-            1 => 'Disable',
-        },
-    },
-    15 => {
+    8 => {
         Name => 'ShutterCurtainSync',
         PrintConv => {
             0 => '1st-curtain sync',
             1 => '2nd-curtain sync',
         },
     },
-    16 => {
-        Name => 'SafetyShiftInAvOrTv',
-        PrintConv => {
-            0 => 'Disable',
-            1 => 'Enable',
-        },
-    },
-    17 => {
-        Name => 'AFPointActivationArea',
-        PrintConv => {
-            0 => 'Single AF Point',
-            1 => 'Expanded (TTL. of 7 AF Points)',
-            2 => 'Automatic Expanded (Max. 13)',
-        },
-    },
-    18 => {
-        Name => 'SwitchToRegisteredAFPoint',
-        PrintConv => {
-            0 => 'Assist + AF',
-            1 => 'Assist',
-            2 => 'Only While Pressing Assist',
-        },
-    },
-    19 => {
+    9 => {
         Name => 'LensAFStopButton',
         PrintConv => {
             0 => 'AF Stop',
-            1 => 'AF Start',
-            2 => 'AE Lock While Metering',
-            3 => 'AF Point: M->Auto/Auto->Ctr',
-            4 => 'AF Mode: ONESHOT<->SERVO',
-            5 => 'IS Start',
+            1 => 'Operate AF',
+            2 => 'Lock AE and start timer',
         },
     },
-    20 => {
-        Name => 'AIServoTrackingSensitivity',
+    10 => {
+        Name => 'FillFlashAutoReduction',
+        PrintConv => \%enableDisable,
+    },
+    11 => {
+        Name => 'MenuButtonReturn',
         PrintConv => {
-            0 => 'Standard',
-            1 => 'Slow',
-            2 => 'Moderately Slow',
-            3 => 'Moderately Fast',
-            4 => 'Fast',
+            0 => 'Top',
+            1 => 'Previous (volatile)',
+            2 => 'Previous',
+        },
+    },
+    12 => {
+        Name => 'SetButtonFunction',
+        PrintConv => {
+            0 => 'Not assigned',
+            1 => 'Change quality',
+            2 => 'Change ISO speed',
+            3 => 'Select parameters',
+        },
+    },
+    13 => {
+        Name => 'SensorCleaning',
+        PrintConv => \%disableEnable,
+    },
+    14 => {
+        Name => 'SuperimposedDisplay',
+        PrintConv => \%onOff,
+    },
+    15 => {
+        Name => 'ShutterReleaseNoCFCard',
+        Description => 'Shutter Release W/O CF Card',
+        PrintConv => {
+            0 => 'Yes',
+            1 => 'No',
         },
     },
 );
 
+# Custom functions for unknown cameras
+%Image::ExifTool::CanonCustom::FuncsUnknown = (
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+    PROCESS_PROC => \&ProcessCanonCustom,
+);
+
+# 1D personal function settings (ref PH)
+%Image::ExifTool::CanonCustom::PersonalFuncs = (
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
+    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    WRITABLE => 1,
+    NOTES => 'Personal function settings for the EOS-1D.',
+    FORMAT => 'int16u',
+    FIRST_ENTRY => 1,
+    1 => { Name => 'PF0CustomFuncRegistration', %convPFn },
+    2 => { Name => 'PF1DisableShootingModes',   %convPFn },
+    3 => { Name => 'PF2DisableMeteringModes',   %convPFn },
+    4 => { Name => 'PF3ManualExposureMetering', %convPFn },
+    5 => { Name => 'PF4ExposureTimeLimits',     %convPFn },
+    6 => { Name => 'PF5ApertureLimits',         %convPFn },
+    7 => { Name => 'PF6PresetShootingModes',    %convPFn },
+    8 => { Name => 'PF7BracketContinuousShoot', %convPFn },
+    9 => { Name => 'PF8SetBracketShots',        %convPFn },
+    10 => { Name => 'PF9ChangeBracketSequence', %convPFn },
+    11 => { Name => 'PF10RetainProgramShift',   %convPFn },
+    #12 => { Name => 'PF11Unused',               %convPFn },
+    #13 => { Name => 'PF12Unused',               %convPFn },
+    14 => { Name => 'PF13DrivePriority',        %convPFn },
+    15 => { Name => 'PF14DisableFocusSearch',   %convPFn },
+    16 => { Name => 'PF15DisableAFAssistBeam',  %convPFn },
+    17 => { Name => 'PF16AutoFocusPointShoot',  %convPFn },
+    18 => { Name => 'PF17DisableAFPointSel',    %convPFn },
+    19 => { Name => 'PF18EnableAutoAFPointSel', %convPFn },
+    20 => { Name => 'PF19ContinuousShootSpeed', %convPFn },
+    21 => { Name => 'PF20LimitContinousShots',  %convPFn },
+    22 => { Name => 'PF21EnableQuietOperation', %convPFn },
+    #23 => { Name => 'PF22Unused',               %convPFn },
+    24 => { Name => 'PF23SetTimerLengths',      %convPFn },
+    25 => { Name => 'PF24LightLCDDuringBulb',   %convPFn },
+    26 => { Name => 'PF25DefaultClearSettings', %convPFn },
+    27 => { Name => 'PF26ShortenReleaseLag',    %convPFn },
+    28 => { Name => 'PF27ReverseDialRotation',  %convPFn },
+    29 => { Name => 'PF28NoQuickDialExpComp',   %convPFn },
+    30 => { Name => 'PF29QuickDialSwitchOff',   %convPFn },
+    31 => { Name => 'PF30EnlargementMode',      %convPFn },
+    32 => { Name => 'PF31OriginalDecisionData', %convPFn },
+);
+
+# 1D personal function values (ref PH)
+%Image::ExifTool::CanonCustom::PersonalFuncValues = (
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
+    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    WRITABLE => 1,
+    FORMAT => 'int16u',
+    FIRST_ENTRY => 1,
+    1 => 'PF1Value',
+    2 => 'PF2Value',
+    3 => 'PF3Value',
+    4 => {
+        Name => 'PF4ExposureTimeMin',
+        RawConv => '$val > 0 ? $val : 0',
+        ValueConv => 'exp(-Image::ExifTool::Canon::CanonEv($val*4)*log(2))*1000/8',
+        ValueConvInv => 'Image::ExifTool::Canon::CanonEvInv(-log($val*8/1000)/log(2))/4',
+        PrintConv => 'Image::ExifTool::Exif::PrintExposureTime($val)',
+        PrintConvInv => 'eval $val',
+    },
+    5 => {
+        Name => 'PF4ExposureTimeMax',
+        RawConv => '$val > 0 ? $val : 0',
+        ValueConv => 'exp(-Image::ExifTool::Canon::CanonEv($val*4)*log(2))*1000/8',
+        ValueConvInv => 'Image::ExifTool::Canon::CanonEvInv(-log($val*8/1000)/log(2))/4',
+        PrintConv => 'Image::ExifTool::Exif::PrintExposureTime($val)',
+        PrintConvInv => 'eval $val',
+    },
+    6 => {
+        Name => 'PF5ApertureMin',
+        RawConv => '$val > 0 ? $val : 0',
+        ValueConv => 'exp(Image::ExifTool::Canon::CanonEv($val*4-32)*log(2)/2)',
+        ValueConvInv => '(Image::ExifTool::Canon::CanonEvInv(log($val)*2/log(2))+32)/4',
+        PrintConv => 'sprintf("%.2g",$val)',
+        PrintConvInv => '$val',
+    },
+    7 => {
+        Name => 'PF5ApertureMax',
+        RawConv => '$val > 0 ? $val : 0',
+        ValueConv => 'exp(Image::ExifTool::Canon::CanonEv($val*4-32)*log(2)/2)',
+        ValueConvInv => '(Image::ExifTool::Canon::CanonEvInv(log($val)*2/log(2))+32)/4',
+        PrintConv => 'sprintf("%.2g",$val)',
+        PrintConvInv => '$val',
+    },
+    8 => 'PF8BracketShots',
+    9 => 'PF19ShootingSpeedLow',
+    10 => 'PF19ShootingSpeedHigh',
+    11 => 'PF20MaxContinousShots',
+    12 => 'PF23ShutterButtonTime',
+    13 => 'PF23FELockTime',
+    14 => 'PF23PostReleaseTime',
+    15 => 'PF25AEMode',
+    16 => 'PF25MeteringMode',
+    17 => 'PF25DriveMode',
+    18 => 'PF25AFMode',
+    19 => 'PF25AFPointSel',
+    20 => 'PF25ImageSize',
+    21 => 'PF25WBMode',
+    22 => 'PF25Parameters',
+    23 => 'PF25ColorMatrix',
+    24 => 'PF27Value',
+);
+
 #------------------------------------------------------------------------------
-# process Canon custom
-# Inputs: 0) ExifTool object reference, 1) reference to directory information
-#         2) pointer to tag table
+# Conversion routines
+# Inputs: 0) value to convert
+sub ConvertPfn($)
+{
+    my $val = shift;
+    return $val ? ($val==1 ? 'On' : "On ($val)") : "Off";
+}
+sub ConvertPfnInv($)
+{
+    my $val = shift;
+    return $1 if $val =~ /(\d+)/;
+    return  1 if $val =~ /on/i;
+    return  0 if $val =~ /off/i;
+    return undef;
+}
+
+#------------------------------------------------------------------------------
+# process Canon custom directory
+# Inputs: 0) ExifTool object ref, 1) dirInfo ref, 2) tag table ref
 # Returns: 1 on success
 sub ProcessCanonCustom($$$)
 {
@@ -652,7 +961,8 @@ sub ProcessCanonCustom($$$)
     my $verbose = $exifTool->Options('Verbose');
 
     # first entry in array must be the size
-    unless (Image::ExifTool::Get16u($dataPt,$offset) == $size) {
+    my $len = Image::ExifTool::Get16u($dataPt,$offset);
+    unless ($len == $size or ($$exifTool{CameraModel}=~/\bD60\b/ and $len+2 == $size)) {
         $exifTool->Warn("Invalid CanonCustom data");
         return 0;
     }
@@ -750,7 +1060,7 @@ Image::ExifTool to read this information.
 
 =head1 AUTHOR
 
-Copyright 2003-2005, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
@@ -766,7 +1076,7 @@ it under the same terms as Perl itself.
 =head1 ACKNOWLEDGEMENTS
 
 Thanks to Christian Koller for his work in decoding the 20D custom
-functions.
+functions, and Rainer Honle for decoding the 5D custom functions.
 
 =head1 SEE ALSO
 
