@@ -16,7 +16,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.13';
+$VERSION = '1.14';
 
 %Image::ExifTool::Casio::Main = (
     WRITE_PROC => \&Image::ExifTool::Exif::WriteExif,
@@ -183,7 +183,6 @@ $VERSION = '1.13';
     },
     0x0014 => {
         Name => 'ISO',
-        Description => 'ISO Speed',
         Priority => 0,
         Writable => 'int16u',
         PrintConv => {
@@ -207,8 +206,7 @@ $VERSION = '1.13';
     },
     0x001d => {
         Name => 'FocalLength',
-        Description => 'Focal Length (mm)',
-        Writable => 'rational32u',
+        Writable => 'rational64u',
         PrintConv => 'sprintf("%.1fmm",$val)',
         PrintConvInv => '$val=~s/mm$//;$val',
     },
@@ -358,7 +356,7 @@ Casio maker notes in EXIF information.
 
 =head1 AUTHOR
 
-Copyright 2003-2005, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
