@@ -33,8 +33,212 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.37';
+$VERSION = '1.39';
 
+# nikon lens ID numbers (ref 8/11)
+my %nikonLensIDs = (
+    Notes => q{
+        The Nikon LensID is constructed as a Composite tag from the raw hex values
+        of 8 other tags: LensIDNumber, LensFStops, MinFocalLength, MaxFocalLength,
+        MaxApertureAtMinFocal, MaxApertureAtMaxFocal, MCUVersion and LensType, in
+        that order.
+    },
+    '01 58 50 50 14 14 02 00' => 'AF Nikkor 50mm f/1.8',
+    '02 42 44 5C 2A 34 02 00' => 'AF Zoom-Nikkor 35-70mm f/3.3-4.5',
+    '02 42 44 5C 2A 34 08 00' => 'AF Zoom-Nikkor 35-70mm f/3.3-4.5',
+    '03 48 5C 81 30 30 02 00' => 'AF Zoom-Nikkor 70-210mm f/4',
+    '04 48 3C 3C 24 24 03 00' => 'AF Nikkor 28mm f/2.8',
+    '05 54 50 50 0C 0C 04 00' => 'AF Nikkor 50mm f/1.4',
+    '06 54 53 53 24 24 06 00' => 'AF Micro-Nikkor 55mm f/2.8',
+    '07 40 3C 62 2C 34 03 00' => 'AF Zoom-Nikkor 28-85mm f/3.5-4.5',
+    '08 40 44 6A 2C 34 04 00' => 'AF Zoom-Nikkor 35-105mm f/3.5-4.5',
+    '09 48 37 37 24 24 04 00' => 'AF Nikkor 24mm f/2.8',
+    '0A 48 8E 8E 24 24 03 00' => 'AF Nikkor 300mm f/2.8 IF-ED',
+    '0B 48 7C 7C 24 24 05 00' => 'AF Nikkor 180mm f/2.8 IF-ED',
+    '0D 40 44 72 2C 34 07 00' => 'AF Zoom-Nikkor 35-135mm f/3.5-4.5',
+    '0E 48 5C 81 30 30 05 00' => 'AF Zoom-Nikkor 70-210mm f/4',
+    '0F 58 50 50 14 14 05 00' => 'AF Nikkor 50mm f/1.8 N',
+    '10 48 8E 8E 30 30 08 00' => 'AF Nikkor 300mm f/4 IF-ED',
+    '11 48 44 5C 24 24 08 00' => 'AF Zoom-Nikkor 35-70mm f/2.8',
+    '12 48 5C 81 30 3C 09 00' => 'AF Nikkor 70-210mm f/4-5.6',
+    '13 42 37 50 2A 34 0B 00' => 'AF Zoom-Nikkor 24-50mm f/3.3-4.5',
+    '14 48 60 80 24 24 0B 00' => 'AF Zoom-Nikkor 80-200mm f/2.8 ED',
+    '15 4C 62 62 14 14 0C 00' => 'AF Nikkor 85mm f/1.8',
+    '17 3C A0 A0 30 30 11 00' => 'Nikkor 500mm f/4 P',
+    '18 40 44 72 2C 34 0E 00' => 'AF Zoom-Nikkor 35-135mm f/3.5-4.5 N',
+    '1A 54 44 44 18 18 11 00' => 'AF Nikkor 35mm f/2',
+    '1B 44 5E 8E 34 3C 10 00' => 'AF Zoom-Nikkor 75-300mm f/4.5-5.6',
+    '1C 48 30 30 24 24 12 00' => 'AF Nikkor 20mm f/2.8',
+    '1D 42 44 5C 2A 34 12 00' => 'AF Zoom-Nikkor 35-70mm f/3.3-4.5 N',
+    '1E 54 56 56 24 24 13 00' => 'AF Micro-Nikkor 60mm f/2.8',
+    '20 48 60 80 24 24 15 00' => 'AF Zoom-Nikkor ED 80-200mm f/2.8',
+    '22 48 72 72 18 18 16 00' => 'AF DC-Nikkor 135mm f/2',
+    '24 48 60 80 24 24 1A 02' => 'AF Zoom-Nikkor ED 80-200mm f/2.8D',
+    '25 48 44 5c 24 24 1B 02' => 'AF Zoom-Nikkor 35-70mm f/2.8D',
+    '25 48 44 5c 24 24 52 02' => 'AF Zoom-Nikkor 35-70mm f/2.8D N',
+    '27 48 8E 8E 24 24 F2 02' => 'AF-I Nikkor 300mm f/2.8D IF-ED',
+    '27 48 8E 8E 24 24 1D 02' => 'AF-I Nikkor 300mm f/2.8D IF-ED',
+    '28 3C A6 A6 30 30 1D 02' => 'AF-I Nikkor 600mm f/4D IF-ED',
+    '2A 54 3C 3C 0C 0C 26 02' => 'AF Nikkor 28mm f/1.4D',
+    '2C 48 6A 6A 18 18 27 02' => 'AF DC-Nikkor 105mm f/2D',
+    '2D 48 80 80 30 30 21 02' => 'AF Micro-Nikkor 200mm f/4D IF-ED',
+    '2E 48 5C 82 30 3C 28 02' => 'AF Nikkor 70-210mm f/4-5.6D',
+    '2F 48 30 44 24 24 29 02' => 'AF Zoom-Nikkor 20-35mm f/2.8(IF)',
+    '31 54 56 56 24 24 25 02' => 'AF Micro-Nikkor 60mm f/2.8D',
+    '32 54 6A 6A 24 24 35 02' => 'AF Micro-Nikkor 105mm f/2.8D',
+    '33 48 2D 2D 24 24 31 02' => 'AF Nikkor 18mm f/2.8D',
+    '34 48 29 29 24 24 32 02' => 'AF Fisheye Nikkor 16mm f/2.8D',
+    '36 48 37 37 24 24 34 02' => 'AF Nikkor 24mm f/2.8D',
+    '37 48 30 30 24 24 36 02' => 'AF Nikkor 20mm f/2.8D',
+    '38 4C 62 62 14 14 37 02' => 'AF Nikkor 85mm f/1.8D',
+    '3B 48 44 5C 24 24 3A 02' => 'AF Zoom-Nikkor 35-70mm f/2.8D N',
+    '3D 3C 44 60 30 3C 3E 02' => 'AF Zoom-Nikkor 35-80mm f/4-5.6D',
+    '3E 48 3C 3C 24 24 3D 02' => 'AF Nikkor 28mm f/2.8D',
+    '41 48 7c 7c 24 24 43 02' => 'AF Nikkor 180mm f/2.8D IF-ED',
+    '42 54 44 44 18 18 44 02' => 'AF Nikkor 35mm f/2D',
+    '43 54 50 50 0C 0C 46 02' => 'AF Nikkor 50mm f/1.4D',
+    '46 3C 44 60 30 3C 49 02' => 'AF Zoom-Nikkor 35-80mm f/4-5.6D N',
+    '47 42 37 50 2A 34 4A 02' => 'AF Zoom-Nikkor 24-50mm f/3.3-4.5D',
+    '48 48 8E 8E 24 24 4B 02' => 'AF-S Nikkor 300mm f/2.8D IF-ED',
+    '4A 54 62 62 0C 0C 4D 02' => 'AF Nikkor 85mm f/1.4D IF',
+    '4C 40 37 6E 2C 3C 4F 02' => 'AF Zoom-Nikkor 24-120mm f/3.5-5.6D IF',
+    '4D 40 3C 80 2C 3C 62 02' => 'AF Zoom-Nikkor 28-200mm f/3.5-5.6D IF',
+    '4E 48 72 72 18 18 51 02' => 'AF DC-Nikkor 135mm f/2D',
+    '4F 40 37 5C 2C 3C 53 06' => 'IX-Nikkor 24-70mm f/3.5-5.6',
+    '53 48 60 80 24 24 60 02' => 'AF Zoom-Nikkor 80-200mm f/2.8D ED',
+    '54 44 5C 7C 34 3C 58 02' => 'AF Zoom-Micro Nikkor 70-180mm f/4.5-5.6D ED',
+    '56 48 5C 8E 30 3C 5A 02' => 'AF Zoom-Nikkor 70-300mm f/4-5.6D ED',
+    '59 48 98 98 24 24 5D 02' => 'AF-S Nikkor 400mm f/2.8D IF-ED',
+    '5A 3C 3E 56 30 3C 5E 06' => 'IX-Nikkor 30-60mm f/4-5.6',
+    '5D 48 3C 5C 24 24 63 02' => 'AF-S Zoom-Nikkor 28-70mm f/2.8D IF-ED',
+    '5E 48 60 80 24 24 64 02' => 'AF-S Zoom-Nikkor 80-200mm f/2.8D IF-ED',
+    '5F 40 3C 6A 2C 34 65 02' => 'AF Zoom-Nikkor 28-105mm f/3.5-4.5D IF',
+    '61 44 5E 86 34 3C 67 02' => 'AF Zoom-Nikkor 75-240mm f/4.5-5.6D',
+    '63 48 2B 44 24 24 68 02' => 'AF-S Nikkor 17-35mm f/2.8D IF-ED',
+    '64 00 62 62 24 24 6A 02' => 'PC Micro-Nikkor 85mm f/2.8D',
+    '65 44 60 98 34 3C 6B 0A' => 'AF VR Zoom-Nikkor 80-400mm f/4.5-5.6D ED',
+    '66 40 2D 44 2C 34 6C 02' => 'AF Zoom-Nikkor 18-35mm f/3.5-4.5D IF-ED',
+    '67 48 37 62 24 30 6D 02' => 'AF Zoom-Nikkor 24-85mm f/2.8-4D IF',
+    '68 42 3C 60 2A 3C 6E 06' => 'AF Zoom-Nikkor 28-80mm f/3.3-5.6G',
+    '69 48 5C 8E 30 3C 6F 06' => 'AF Zoom-Nikkor 70-300mm f/4-5.6G',
+    '6A 48 8E 8E 30 30 70 02' => 'AF-S Nikkor 300mm f/4D IF-ED',
+    '6B 48 24 24 24 24 71 02' => 'AF Nikkor ED 14mm f/2.8D',
+    '6D 48 8E 8E 24 24 73 02' => 'AF-S Nikkor 300mm f/2.8D IF-ED II',
+    '6E 48 98 98 24 24 74 02' => 'AF-S Nikkor 400mm f/2.8D IF-ED II',
+    '6F 3C A0 A0 30 30 75 02' => 'AF-S Nikkor 500mm f/4D IF-ED',
+    '70 3C A6 A6 30 30 76 02' => 'AF-S Nikkor 600mm f/4D IF-ED',
+    '72 48 4C 4C 24 24 77 00' => 'Nikkor 45mm f/2.8 P',
+    '74 40 37 62 2C 34 78 06' => 'AF-S Zoom-Nikkor 24-85mm f/3.5-4.5G IF-ED',
+    '75 40 3C 68 2C 3C 79 06' => 'AF Zoom-Nikkor 28-100mm f/3.5-5.6G',
+    '76 58 50 50 14 14 7A 02' => 'AF Nikkor 50mm f/1.8D',
+    '77 48 5C 80 24 24 7B 0E' => 'AF-S VR Zoom-Nikkor 70-200mm f/2.8G IF-ED',
+    '78 40 37 6E 2C 3C 7C 0E' => 'AF-S VR Zoom-Nikkor 24-120mm f/3.5-5.6G IF-ED',
+    '79 40 3C 80 2C 3C 7F 06' => 'AF Zoom-Nikkor 28-200mm f/3.5-5.6G IF-ED',
+    '7A 3C 1F 37 30 30 7E 06' => 'AF-S DX Zoom-Nikkor 12-24mm f/4G IF-ED',
+    '7B 48 80 98 30 30 80 0E' => 'AF-S VR Zoom-Nikkor 200-400mm f/4G IF-ED',
+    '7D 48 2B 53 24 24 82 06' => 'AF-S DX Zoom-Nikkor 17-55mm f/2.8G IF-ED',
+    '7F 40 2D 5C 2C 34 84 06' => 'AF-S DX Zoom-Nikkor 18-70mm f/3.5-4.5G IF-ED',
+    '80 48 1A 1A 24 24 85 06' => 'AF DX Fisheye-Nikkor 10.5mm f/2.8G ED',
+    '81 54 80 80 18 18 86 0E' => 'AF-S VR Nikkor 200mm f/2G IF-ED',
+    '82 48 8E 8E 24 24 87 0E' => 'AF-S VR Nikkor 300mm f/2.8G IF-ED',
+    '89 3C 53 80 30 3C 8B 06' => 'AF-S DX Zoom-Nikkor 55-200mm f/4-5.6G ED',
+    '8B 40 2D 80 2C 3C 8D 0E' => 'AF-S DX VR Zoom-Nikkor 18-200mm f/3.5-5.6G IF-ED',
+    '8C 40 2D 53 2C 3C 8E 06' => 'AF-S DX Zoom-Nikkor 18-55mm f/3.5-5.6G ED',
+#
+    '06 3F 68 68 2C 2C 06 00' => 'Cosina 100mm F/3.5 Macro',
+#
+    '26 48 11 11 30 30 1C 02' => 'Sigma 8mm F4 EX Circular Fisheye',
+    '48 3C 19 31 30 3C 4B 06' => 'Sigma 10-20mm F4-5.6 EX DC HSM',
+    '48 38 1F 37 34 3C 4B 06' => 'Sigma 12-24mm F4.5-5.6 EX Aspherical DG HSM',
+    '02 3F 24 24 2C 2C 02 00' => 'Sigma 14mm F3.5',
+    '48 48 24 24 24 24 4B 02' => 'Sigma 14mm F2.8 EX ASPHERICAL HSM',
+    '26 48 27 27 24 24 1C 02' => 'Sigma 15mm F2.8 EX Diagonal Fish-Eye',
+    '26 40 27 3F 2C 34 1C 02' => 'Sigma 15-30mm F3.5-4.5 EX Aspherical DG DF',
+    '48 48 2B 44 24 30 4B 06' => 'Sigma 17-35mm F2.8-4 EX DG  Aspherical HSM',
+    '26 54 2B 44 24 30 1C 02' => 'Sigma 17-35mm F2.8-4 EX ASPHERICAL',
+    '26 48 2D 50 24 24 1C 06' => 'Sigma 18-50mm F2.8 EX DC',
+    '26 40 2D 50 2C 3C 1C 06' => 'Sigma 18-50mm F3.5-5.6 DC',
+    '26 40 2D 70 2B 3C 1C 06' => 'Sigma 18-125mm F3.5-5.6 DC',
+    '26 40 2D 80 2C 40 1C 06' => 'Sigma 18-200mm F3.5-6.3 DC',
+    '26 58 31 31 14 14 1C 02' => 'Sigma 20mm F1.8 EX Aspherical DG DF RF',
+    '26 58 37 37 14 14 1C 02' => 'Sigma 24mm F1.8 EX Aspherical DG DF MACRO',
+    '02 46 37 37 25 25 02 00' => 'Sigma 24mm F2.8 Macro',
+    '26 48 37 56 24 24 1C 02' => 'Sigma 24-60mm F2.8 EX DG',
+    '26 54 37 5C 24 24 1C 02' => 'Sigma 24-70mm F2.8 EX DG Macro',
+    '26 40 37 5C 2C 3C 1C 02' => 'Sigma 24-70mm F3.5-5.6 ASPHERICAL HF',
+    '26 54 37 73 24 34 1C 02' => 'Sigma 24-135mm F2.8-4.5',
+    '26 58 3C 3C 14 14 1C 02' => 'Sigma 28mm F1.8 EX DG DF',
+    '26 48 3C 5C 24 24 1C 06' => 'Sigma 28-70mm F2.8 EX DG',
+    '26 48 3C 5C 24 30 1C 02' => 'Sigma 28-70mm F2.8-4 HIGH SPEED ZOOM',
+    '02 46 3C 5C 25 25 02 00' => 'Sigma 28-70mm F2.8',
+    '02 3F 3C 5C 2D 35 02 00' => 'Sigma 28-70mm F3.5-4.5 UC',
+    '26 40 3C 60 2C 3C 1C 02' => 'Sigma 28-80mm F3.5-5.6 Mini Zoom Macro II Aspherical',
+    '26 3E 3C 6A 2E 3C 1C 02' => 'Sigma 28-105mm F3.8-5.6 UC-III ASPHERICAL IF',
+    '26 40 3C 80 2C 3C 1C 02' => 'Sigma 28-200mm F3.5-5.6 Compact Aspherical Hyperzoom Macro',
+    '26 40 3C 80 2B 3C 1C 02' => 'Sigma 28-200mm F3.5-5.6 Compact Aspherical Hyperzoom Macro',
+    '26 41 3C 8E 2C 40 1C 02' => 'Sigma 28-300mm F3.5-6.3 DG MACRO',
+    '26 40 3C 8E 2C 40 1C 02' => 'Sigma 28-300mm F3.5-6.3 Macro',
+    '48 54 3E 3E 0C 0C 4B 06' => 'Sigma 30mm F1.4 EX DC HSM',
+    '02 40 44 73 2B 36 02 00' => 'Sigma 35-135mm F3.5-4.5 a',
+    '32 54 50 50 24 24 35 02' => 'Sigma 50mm F2.8 EX DG Macro',
+    '48 3C 50 A0 30 40 4B 02' => 'Sigma 50-500mmF4-6.3 EX APO RF HSM',
+    '26 3C 54 80 30 3C 1C 06' => 'Sigma 55-200mm F4-5.6 DC',
+    '48 54 5C 80 24 24 4B 02' => 'Sigma 70-200mm F2.8 EX APO IF HSM',
+    '26 3C 5C 8E 30 3C 1C 02' => 'Sigma 70-300mm F4-5.6 DG MACRO',
+    '56 3C 5C 8E 30 3C 1C 02' => 'Sigma 70-300mm F4-5.6 APO Macro Super II',
+    '02 37 5E 8E 35 3D 02 00' => 'Sigma 75-300mm F4.5-5.6 APO',
+    '02 48 65 65 24 24 02 00' => 'Sigma 90mm F2.8 Macro',
+    '77 44 61 98 34 3C 7B 0E' => 'Sigma 80-400mm f4.5-5.6 EX OS',
+    '48 48 68 8E 30 30 4B 02' => 'Sigma 100-300mm F4 EX IF HSM',
+    '26 44 73 98 34 3C 1C 02' => 'Sigma 135-400mm F4.5-5.6 APO Aspherical',
+    '48 48 76 76 24 24 4B 06' => 'Sigma 150mm F2.8 EX DG APO Macro HSM',
+    '26 40 7B A0 34 40 1C 02' => 'Sigma APO 170-500mm F5-6.3 ASPHERICAL RF',
+    '48 4C 7D 7D 2C 2C 4B 02' => 'Sigma APO MACRO 180mm F3.5 EX DG HSM',
+    '48 54 8E 8E 24 24 4B 02' => 'Sigma APO 300mm F2.8 EX DG HSM',
+    '26 48 8E 8E 30 30 1C 02' => 'Sigma APO TELE MACRO 300mm F4',
+    '02 2F 98 98 3D 3D 02 00' => 'Sigma 400mm F5.6 APO',
+#
+    '03 43 5C 81 35 35 02 00' => 'Soligor AF C/D ZOOM UMCS 70-210mm 1:4.5',
+#
+    '00 3C 1F 37 30 30 00 06' => 'Tokina AT-X 124 AF PRO DX - AF 12-24mm f/4',
+    '00 40 2B 2B 2C 2C 00 02' => 'Tokina AT-X 17 AF PRO - AF 17mm f/3.5',
+    '25 48 3C 5C 24 24 1B 02' => 'Tokina AT-X287AF PRO SV 28-70mm F2.8',
+    '00 48 3C 60 24 24 00 02' => 'Tokina AT-X280AF PRO 28-80mm F2.8 ASPHERICAL',
+    '14 54 60 80 24 24 0B 00' => 'Tokina AT-X828AF 80-200mm F2.8',
+    '24 44 60 98 34 3C 1A 02' => 'Tokina AT-X840AF II 80-400mm F4.5-5.6',
+    '00 54 68 68 24 24 00 02' => 'Tokina AT-X M100 PRO D - 100mm F2.8',
+    '14 48 68 8E 30 30 0B 00' => 'Tokina AT-X340AF II 100-300mm F4',
+    '00 54 8E 8E 24 24 00 02' => 'Tokina AT-X300AF PRO 300mm F2.8',
+#
+    '00 36 1C 2D 34 3C 00 06' => 'Tamron SP AF11-18mm F/4.5-5.6 Di II LD Aspherical (IF)',
+    '07 46 2B 44 24 30 03 02' => 'Tamron SP AF17-35mm F/2.8-4 Di LD Aspherical (IF)',
+    '00 3F 2D 80 2B 40 00 06' => 'Tamron AF18-200mm f/3.5-6.3 XR Di II LD Aspherical (IF)',
+    '00 3F 2D 80 2C 40 00 06' => 'Tamron AF18-200mm f/3.5-6.3 XR Di II LD Aspherical (IF) MACRO',
+    '07 40 30 45 2D 35 03 02' => 'Tamron AF19-35mm F3.5-4.5',
+    '00 49 30 48 22 2B 00 02' => 'Tamron SP AF20-40mm f/2.7-3.5', 
+    '45 41 37 72 2C 3C 48 02' => 'Tamron SP AF24-135mm F3.5-5.6 AD ASPHERICAL(IF)MACRO',
+    '33 54 3C 5E 24 24 62 02' => 'Tamron SP AF28-75mm F2.8 XR Di LD ASPHERICAL(IF)MACRO',
+    '10 3D 3C 60 2C 3C D2 02' => 'Tamron AF28-80mm f/3.5-5.6 Aspherical',
+    '45 3D 3C 60 2C 3C 48 02' => 'Tamron AF 28-80mm F3.5-5.6 ASPHERICAL', 
+    '00 48 3C 6A 24 24 00 02' => 'Tamron SP AF28-105mm f/2.8',
+    '0B 3E 3D 7F 2F 3D 0E 02' => 'Tamron AF28-200mm f/3.8-5.6D',
+    '0B 3E 3D 7F 2F 3D 0E 00' => 'Tamron AF28-200mm f/3.8-5.6',
+    '4D 41 3C 8E 2B 40 62 02' => 'Tamron AF28-300mm f/3.5-6.3 XR Di LD Aspherical (IF)',
+    '4D 41 3C 8E 2C 40 62 02' => 'Tamron AF28-300mm f/3.5-6.3D',
+    '69 48 5C 8E 30 3C 6F 02' => 'Tamron AF70-300mm F4-5.6 LD MACRO 1:2',
+    '32 53 64 64 24 24 35 02' => 'Tamron SP AF90mm f/2.8 Di 1:1 Macro',
+    '00 4C 7C 7C 2C 2C 00 02' => 'Tamron SP AF180mm F3.5 Di Model B01',
+    '20 3C 80 98 3D 3D 1E 02' => 'Tamron AF200-400mm f/5.6 LD IF',
+    '00 3E 80 A0 38 3F 00 02' => 'Tamron SP AF200-500mm f/5-6.3 Di LD (IF)',
+    '00 3F 80 A0 38 3F 00 02' => 'Tamron SP AF200-500mm F5-6.3 Di',
+#
+    '00 00 00 00 00 00 00 01' => 'Manual Lens No CPU',
+    '1E 5D 64 64 20 20 13 00' => 'Unknown 90mm F/2.5',
+    '2F 40 30 44 2C 34 29 02' => 'Unknown 20-35mm F/3.5-4.5D',
+    '12 3B 68 8D 3D 43 09 02' => 'Unknown 100-290mm F5.6-6.7',
+);
+
+# Nikon maker note tags
 %Image::ExifTool::Nikon::Main = (
     PROCESS_PROC => \&Image::ExifTool::Nikon::ProcessNikon,
     WRITE_PROC => \&Image::ExifTool::Exif::WriteExif,
@@ -58,7 +262,6 @@ $VERSION = '1.37';
         # this is the ISO actually used by the camera
         # (may be different than ISO setting if auto)
         Name => 'ISO',
-        Description => 'ISO Speed',
         Writable => 'int16u',
         Priority => 0,  # the EXIF ISO is more reliable
         Count => 2,
@@ -82,7 +285,7 @@ $VERSION = '1.37';
     0x000b => { Name => 'WhiteBalanceFineTune', Writable => 'int16u' }, #2
     0x000c => {
         Name => 'ColorBalance1',
-        Writable => 'rational32u',
+        Writable => 'rational64u',
         Count => 4,
     },
     0x000e => {
@@ -142,7 +345,7 @@ $VERSION = '1.37';
     },
     0x0019 => { #5
         Name => 'ExposureBracketValue',
-        Format => 'rational32s',
+        Format => 'rational64s',
         PrintConv => 'Image::ExifTool::Exif::ConvertFraction($val)',
     },
     0x001d => { #4
@@ -158,7 +361,7 @@ $VERSION = '1.37';
         Name => 'LensType',
         Writable => 'int8u',
         # credit to Tom Christiansen (ref 7) for figuring this out...
-        PrintConv => q[$_ = $val ? Image::ExifTool::Exif::DecodeBits($val,
+        PrintConv => q[$_ = $val ? Image::ExifTool::DecodeBits($val,
             {
                 0 => 'MF',
                 1 => 'D',
@@ -179,7 +382,7 @@ $VERSION = '1.37';
     },
     0x0084 => { #2
         Name => "Lens",
-        Writable => 'rational32u',
+        Writable => 'rational64u',
         Count => 4,
         # short focal, long focal, aperture at short focal, aperture at long focal
         PrintConv => q{
@@ -191,11 +394,11 @@ $VERSION = '1.37';
     },
     0x0085 => {
         Name => 'ManualFocusDistance',
-        Writable => 'rational32u',
+        Writable => 'rational64u',
     },
     0x0086 => {
         Name => 'DigitalZoom',
-        Writable => 'rational32u',
+        Writable => 'rational64u',
     },
     0x0087 => { #5
         Name => 'FlashMode',
@@ -253,7 +456,7 @@ $VERSION = '1.37';
                 return 'Single-Frame' unless $val;
                 $_ = 'Single-Frame, ';
             }
-            return $_ . Image::ExifTool::Exif::DecodeBits($val,
+            return $_ . Image::ExifTool::DecodeBits($val,
             {
                 0 => 'Continuous',
                 1 => 'Delay',
@@ -395,7 +598,7 @@ $VERSION = '1.37';
     },
     0x009a => { #10
         Name => 'SensorPixelSize',
-        Writable => 'rational32u',
+        Writable => 'rational64u',
         Count => 2,
         PrintConv => '$val=~s/ / x /;"$val um"',
         PrintConvInv => '$val=~tr/a-zA-Z/ /;$val',
@@ -467,18 +670,9 @@ $VERSION = '1.37';
     FIRST_ENTRY => 0,
     GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
     0 => {
-        Name => 'RedBalance',
-        ValueConv => '$val / 256',
-        ValueConvInv => '$val * 256',
-        PrintConv => 'sprintf("%.5f",$val)',
-        PrintConvInv => '$val',
-    },
-    1 => {
-        Name => 'BlueBalance',
-        ValueConv => '$val / 256',
-        ValueConvInv => '$val * 256',
-        PrintConv => 'sprintf("%.5f",$val)',
-        PrintConvInv => '$val',
+        Name => 'WB_RBGGLevels',
+        Format => 'int16u[4]',
+        Protected => 1,
     },
 );
 
@@ -488,20 +682,13 @@ $VERSION = '1.37';
     WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
     CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
     WRITABLE => 1,
-    FORMAT => 'rational16s',
+    FORMAT => 'int16u',
     FIRST_ENTRY => 0,
     GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
     0 => {
-        Name => 'RedBalance',
-        PrintConv => 'sprintf("%.5f",$val)',
-        PrintConvInv => '$val',
-    },
-    1 => {
-        Name => 'BlueBalance',
-        ValueConv => '$val!=0 ? 1/$val : 0',
-        ValueConvInv => '$val!=0 ? 1/$val : 0',
-        PrintConv => 'sprintf("%.5f",$val)',
-        PrintConvInv => '$val',
+        Name => 'WB_RGGBLevels',
+        Format => 'int16u[4]',
+        Protected => 1,
     },
 );
 
@@ -511,18 +698,13 @@ $VERSION = '1.37';
     WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
     CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
     WRITABLE => 1,
-    FORMAT => 'rational16s',
+    FORMAT => 'int16u',
     FIRST_ENTRY => 0,
     GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
     0 => {
-        Name => 'RedBalance',
-        PrintConv => 'sprintf("%.5f",$val)',
-        PrintConvInv => '$val',
-    },
-    1 => {
-        Name => 'BlueBalance',
-        PrintConv => 'sprintf("%.5f",$val)',
-        PrintConvInv => '$val',
+        Name => 'WB_RGBGLevels',
+        Format => 'int16u[4]',
+        Protected => 1,
     },
 );
 
@@ -607,111 +789,13 @@ my %nikonApertureConversions = (
     PrintConv => 'sprintf("%.1f",$val)',
     PrintConvInv => '$val',
 );
+
 my %nikonFocalConversions = (
     ValueConv => '5 * 2**($val/24)',
     ValueConvInv => '$val>0 ? 24*log($val/5)/log(2) : 0',
     PrintConv => 'sprintf("%.1fmm",$val)',
     PrintConvInv => '$val=~s/\s*mm$//;$val',
 );
-# nikon lens ID numbers (ref 8/11)
-my %nikonLensIDs = (
-    0 => 'Unknown Nikkor or Tokina',
-    1 => 'AF Nikkor 50mm f/1.8',
-    2 => 'AF Zoom-Nikkor 35-70mm f/3.3-4.5 or Sigma non-D',
-    3 => 'AF Zoom-Nikkor 70-210mm f/4 or Soligor',
-    4 => 'AF Nikkor 28mm f/2.8',
-    5 => 'AF Nikkor 50mm f/1.4',
-    6 => 'AF Micro-Nikkor 55mm f/2.8 or Cosina',
-    7 => 'AF Zoom-Nikkor 28-85mm f/3.5-4.5 or Tamron',
-    8 => 'AF Zoom-Nikkor 35-105mm f/3.5-4.5',
-    9 => 'AF Nikkor 24mm f/2.8',
-    10 => 'AF Nikkor 300mm f/2.8 IF-ED',
-    11 => 'AF Nikkor 180mm f/2.8 IF-ED or Tamron',
-    13 => 'AF Zoom-Nikkor 35-135mm f/3.5-4.5',
-    14 => 'AF Zoom-Nikkor 70-210mm f/4',
-    15 => 'AF Nikkor 50mm f/1.8 N',
-    16 => 'AF Nikkor 300mm f/4 IF-ED',
-    17 => 'AF Zoom-Nikkor 35-70mm f/2.8',
-    18 => 'AF Nikkor 70-210mm f/4-5.6',
-    19 => 'AF Zoom-Nikkor 24-50mm f/3.3-4.5',
-    20 => 'AF Zoom-Nikkor 80-200mm f/2.8 ED',
-    21 => 'AF Nikkor 85mm f/1.8',
-    23 => 'Nikkor 500mm f/4 P',
-    24 => 'AF Zoom-Nikkor 35-135mm f/3.5-4.5 N',
-    26 => 'AF Nikkor 35mm f/2',
-    27 => 'AF Zoom-Nikkor 75-300mm f/4.5-5.6',
-    28 => 'AF Nikkor 20mm f/2.8',
-    29 => 'AF Zoom-Nikkor 35-70mm f/3.3-4.5 N',
-    30 => 'AF Micro-Nikkor 60mm f/2.8 or third party lens',
-    32 => 'Unknown Nikkor or Tamron',
-    36 => 'AF Zoom-Nikkor ED 80-200mm f/2.8D',
-    37 => 'AF Zoom-Nikkor 35-70mm f/2.8D N',
-    38 => 'Unknown Nikkor or Sigma D',
-    39 => 'AF-I Nikkor 300mm f/2.8D IF-ED',
-    42 => 'AF Nikkor 28mm f/1.4D',
-    44 => 'AF DC-Nikkor 105mm f/2D',
-    45 => 'AF Micro-Nikkor 200mm f/4D IF-ED',
-    46 => 'AF Nikkor 70-210mm f/4-5.6D',
-    47 => 'Unknown Nikkor or third party lens',
-    49 => 'AF Micro-Nikkor 60mm f/2.8D',
-    50 => 'AF Micro-Nikkor 105mm f/2.8D or Sigma Macro D',
-    51 => 'AF Nikkor 18mm f/2.8D',
-    52 => 'Unknown Nikkor or Tamron',
-    54 => 'AF Nikkor 24mm f/2.8D',
-    55 => 'AF Nikkor 20mm f/2.8D',
-    56 => 'AF Nikkor 85mm f/1.8D',
-    59 => 'AF Zoom-Nikkor 35-70mm f/2.8D N',
-    61 => 'AF Zoom-Nikkor 35-80mm f/4-5.6D',
-    62 => 'AF Nikkor 28mm f/2.8D',
-    65 => 'AF Nikkor 180mm f/2.8D IF-ED',
-    66 => 'AF Nikkor 35mm f/2D',
-    67 => 'AF Nikkor 50mm f/1.4D',
-    70 => 'AF Zoom-Nikkor 35-80mm f/4-5.6D N',
-    72 => 'AF-S Nikkor 300mm f/2.8D IF-ED or Sigma HSM',
-    74 => 'AF Nikkor 85mm f/1.4D IF',
-    76 => 'AF Zoom-Nikkor 24-120mm f/3.5-5.6D IF',
-    77 => 'AF Zoom-Nikkor 28-200mm f/3.5-5.6D IF or Tamron',
-    78 => 'AF DC-Nikkor 135mm f/2D',
-    79 => 'IX-Nikkor 24-70mm f/3.5-5.6',
-    83 => 'AF Zoom-Nikkor 80-200mm f/2.8D ED',
-    84 => 'AF Zoom-Micro Nikkor 70-180mm f/4.5-5.6D ED',
-    86 => 'AF Zoom-Nikkor 70-300mm f/4-5.6D ED or Sigma D',
-    89 => 'AF-S Nikkor 400mm f/2.8D IF-ED',
-    90 => 'IX-Nikkor 30-60mm f/4-5.6',
-    93 => 'AF-S Zoom-Nikkor 28-70mm f/2.8D IF-ED',
-    94 => 'AF-S Zoom-Nikkor 80-200mm f/2.8D IF-ED',
-    95 => 'AF Zoom-Nikkor 28-105mm f/3.5-4.5D IF',
-    97 => 'AF Zoom-Nikkor 75-240mm f/4.5-5.6D',
-    99 => 'AF-S Nikkor 17-35mm f/2.8D IF-ED',
-    100 => 'PC Micro-Nikkor 85mm f/2.8D',
-    101 => 'AF VR Zoom-Nikkor 80-400mm f/4.5-5.6D ED',
-    102 => 'AF Zoom-Nikkor 18-35mm f/3.5-4.5D IF-ED',
-    103 => 'AF Zoom-Nikkor 24-85mm f/2.8-4D IF',
-    104 => 'AF Zoom-Nikkor 28-80mm f/3.3-5.6G',
-    105 => 'AF Zoom-Nikkor 70-300mm f/4-5.6G',
-    106 => 'AF-S Nikkor 300mm f/4D IF-ED',
-    109 => 'AF-S Nikkor 300mm f/2.8D IF-ED II',
-    110 => 'AF-S Nikkor 400mm f/2.8D IF-ED II',
-    111 => 'AF-S Nikkor 500mm f/4D IF-ED',
-    112 => 'AF-S Nikkor 600mm f/4D IF-ED',
-    114 => 'Nikkor 45mm f/2.8 P',
-    116 => 'AF-S Zoom-Nikkor 24-85mm f/3.5-4.5G IF-ED',
-    117 => 'AF Zoom-Nikkor 28-100mm f/3.5-5.6G',
-    118 => 'AF Nikkor 50mm f/1.8D',
-    119 => 'AF-S VR Zoom-Nikkor 70-200mm f/2.8G IF-ED or Sigma OS',
-    120 => 'AF-S VR Zoom-Nikkor 24-120mm f/3.5-5.6G IF-ED',
-    121 => 'AF Zoom-Nikkor 28-200mm f/3.5-5.6G IF-ED',
-    122 => 'AF-S DX Zoom-Nikkor 12-24mm f/4G IF-ED',
-    123 => 'AF-S VR Zoom-Nikkor 200-400mm f/4G IF-ED',
-    125 => 'AF-S DX Zoom-Nikkor 17-55mm f/2.8G IF-ED',
-    127 => 'AF-S DX Zoom-Nikkor 18-70mm f/3.5-4.5G IF-ED',
-    128 => 'AF DX Fisheye-Nikkor 10.5mm f/2.8G ED',
-    129 => 'AF-S VR Nikkor 200mm f/2G IF-ED',
-    130 => 'AF-S VR Nikkor 300mm f/2.8G IF-ED',
-    137 => 'AF-S DX Zoom-Nikkor 55-200mm f/4-5.6G ED',
-    139 => 'AF-S DX VR Zoom-Nikkor 18-200mm f/3.5-5.6G IF-ED',#8/10
-    140 => 'AF-S DX Zoom-Nikkor 18-55mm f/3.5-5.6G ED',
-);
 
 # Version 100 Nikon lens data
 %Image::ExifTool::Nikon::LensData00 = (
@@ -727,8 +811,8 @@ my %nikonLensIDs = (
         Format => 'undef[4]',
     },
     0x06 => { #8
-        Name => 'LensID',
-        PrintConv => \%nikonLensIDs,
+        Name => 'LensIDNumber',
+        Notes => 'see LensID values below',
     },
     0x07 => { #8
         Name => 'LensFStops',
@@ -789,8 +873,8 @@ which make writing more difficult.
         %nikonFocalConversions,
     },
     0x0b => { #8
-        Name => 'LensID',
-        PrintConv => \%nikonLensIDs,
+        Name => 'LensIDNumber',
+        Notes => 'see LensID values below',
     },
     0x0c => { #8
         Name => 'LensFStops',
@@ -832,7 +916,23 @@ which make writing more difficult.
             1 => 'Nikon:LensType',
         },
         ValueConv => '"$val[0] $val[1]"',
-        PrintConv => '"$valPrint[0] $valPrint[1]"',
+        PrintConv => '"$prt[0] $prt[1]"',
+    },
+    LensID => {
+        SeparateTable => 'Nikon',   # print values in a separate table
+        Require => {
+            0 => 'Nikon:LensIDNumber',
+            1 => 'LensFStops',
+            2 => 'MinFocalLength',
+            3 => 'MaxFocalLength',
+            4 => 'MaxApertureAtMinFocal',
+            5 => 'MaxApertureAtMaxFocal',
+            6 => 'MCUVersion',
+            7 => 'Nikon:LensType',
+        },
+        # construct lens ID string as per ref 11
+        ValueConv => 'uc(join(" ",(unpack("H*",pack("C*",@raw)) =~ /../g)))',
+        PrintConv => \%nikonLensIDs,
     },
 );
 
@@ -1070,7 +1170,7 @@ Nikon maker notes in EXIF information.
 
 =head1 AUTHOR
 
-Copyright 2003-2005, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2006, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
